@@ -1,40 +1,46 @@
 #' Statistical process control charts.
-#' 
-#' The \code{qic()} function creates run charts and Shewhart control charts for 
+#'
+#' The \code{qic()} function creates run charts and Shewhart control charts for
 #' process control and improvement. Included control charts are: I, MR, Xbar, S,
 #' T, C, U, U', P, P', and G charts.
+#'
+#' Non-random variation in the form of minor to moderate persistent shifts in
+#' data over time is (by default) identified by the Anhoej rules for unusually
+#' long runs and unusually few crossing. Use the EXPERIMENTAL method argument to
+#' apply the bestbox or cutbox methods as described in Anhøj & Wentzel-Larsen
+#' (2020) \url{https://doi.org/10.1371/journal.pone.0233920}.
 #' 
-#' Non-random variation in the form of minor to moderate persistens shifts in 
-#' data over time is identified by the Anhoej rules for unusually long runs and 
-#' unusually few crossing. Special cause variation in the form of larger, 
-#' possibly transient, shifts in data is identified by Shewhart's 3-sigma rule.
-#' 
+#' Special cause variation in the form of larger, possibly
+#' transient, shifts in data is identified by Shewhart's 3-sigma rule.
+#'
 #' @param x Vector of subgroup values to plot along the x axis.
 #' @param y Vector of measures or counts to plot on the y axis (numerator).
 #' @param n Vector of subgroup sizes (denominator).
 #' @param data Data frame containing variables used in the plot.
 #' @param notes Character vector of notes to be added to individual data points.
 #' @param facets One or two sided formula with factors used for facetting plots.
-#' @param chart Character value indicating the chart type. Possible values are: 
-#'   'run' (default), 'i', 'mr', 'xbar', 't', 's', 'c', 'u', 'up', 'p', 'pp', 
+#' @param chart Character value indicating the chart type. Possible values are:
+#'   'run' (default), 'i', 'mr', 'xbar', 't', 's', 'c', 'u', 'up', 'p', 'pp',
 #'   and 'g'.
 #' @param agg.fun Aggregate function for summarising the y variable if there are
-#'   more than one observation per subgroup. Only relevant for run charts and I 
+#'   more than one observation per subgroup. Only relevant for run charts and I
 #'   charts. Possible values are: 'mean' (default), 'median', 'sum', and 'sd'.
-#' @param multiply Number indicating a number to multiply y axis by, e.g. 100 
+#' @param method Character indicating the method used for runs analysis. One of
+#'   'anhoej' (default), 'bestbox', and 'cutbox'. EXPERIMENTAL!
+#' @param multiply Number indicating a number to multiply y axis by, e.g. 100
 #'   for percents rather than proportions. See also \code{y.percent} argument.
-#' @param freeze Integer indicating the last data point to include in 
-#'   calculation of baseline paramenters for centre and control lines. Ignored 
-#'   if part argument is given.
+#' @param freeze Integer indicating the last data point to include in
+#'   calculation of baseline parameters for centre and control lines. Ignored if
+#'   part argument is given.
 #' @param part Either integer vector indicating data points before recalculation
 #'   of centre and control lines or character vector indicating chart parts.
-#' @param exclude Integer vector indicating data points to exclude from 
+#' @param exclude Integer vector indicating data points to exclude from
 #'   calculations of centre and control lines.
-#' @param target Numeric, either a single value indicating a target value to be plotted as a 
-#'   horizontal line or a vector for variable target line.
-#' @param cl Numeric, either a single value indicating the centre line if known in 
-#'   advance or a vector for variable centre line.
-#' @param nrow,ncol Number indicating the preferred number of rows and columns 
+#' @param target Numeric, either a single value indicating a target value to be
+#'   plotted as a horizontal line or a vector for variable target line.
+#' @param cl Numeric, either a single value indicating the centre line if known
+#'   in advance or a vector for variable centre line.
+#' @param nrow,ncol Number indicating the preferred number of rows and columns
 #'   in facets.
 #' @param scales Character string, one of 'fixed' (default), 'free_y', 'free_x',
 #'   or 'free' indicating whether y and x axis scales should be the same for all
@@ -44,23 +50,24 @@
 #' @param ylab Character string specifying the y axis label.
 #' @param subtitle Character string specifying the subtitle.
 #' @param caption Character string specifying the caption.
-#' @param part.labels Character vector specifying  labels for 
-#'   chart parts created with the freeze or part argument.
+#' @param part.labels Character vector specifying  labels for chart parts
+#'   created with the freeze or part argument.
 #' @param show.labels Logical indicating whether to show labels for centre and
 #'   control lines on chart. Defaults to TRUE when facets argument is NULL.
 #' @param decimals Integer indicating the preferred number of decimals in centre
 #'   and control line labels.
 #' @param point.size Number specifying the size of data points.
-#' @param x.period Character string specifying the interval cut points of 
-#'   datetime x values used for aggregating y values by week, month, etc.
-#'   See the breaks argument of \code{?cut.POSIXt()} for possible values.
-#' @param x.format Date format of x axis labels. See \code{?strftime()} for 
+#' @param x.period Character string specifying the interval cut points of
+#'   datetime x values used for aggregating y values by week, month, etc. See
+#'   the breaks argument of \code{?cut.POSIXt()} for possible values.
+#' @param x.format Date format of x axis labels. See \code{?strftime()} for
 #'   possible date formats.
 #' @param x.angle Number indicating the angle of x axis labels.
-#' @param x.pad Number indicating expansion of x axis to make room for axis labels.
-#' @param y.expand Numeric value to include in y axis. Useful e.g. for starting 
+#' @param x.pad Number indicating expansion of x axis to make room for axis
+#'   labels.
+#' @param y.expand Numeric value to include in y axis. Useful e.g. for starting
 #'   the y axis at zero.
-#' @param y.neg If TRUE (default), the y axis is allowed to be negative (only 
+#' @param y.neg If TRUE (default), the y axis is allowed to be negative (only
 #'   relevant for I and Xbar charts).
 #' @param y.percent If TRUE, formats y axis labels as percentages.
 #' @param show.grid If TRUE, shows grid.
@@ -68,27 +75,27 @@
 #' @param strip.horizontal If TRUE, makes y strip horizontal.
 #' @param print.summary If TRUE, prints summary.
 # @param ... Additional arguments to plot function.
-#'   
+#'
 #' @return A \code{qic} object. Inherits from 'ggplot'.
-#'   
-#' @seealso \code{vignette('qic')}
-#'   
+#'
+#' @seealso \code{vignette('qic')}.
+#'
 #' @examples
 #' # Lock random number generator to make reproducible results.
 #' set.seed(2)
-#' 
+#'
 #' # Generate vector of 24 random normal numbers
 #' y <- rnorm(24)
-#' 
+#'
 #' # Run chart
 #' qic(y)
-#' 
+#'
 #' # I control chart
 #' qic(y, chart = 'i')
-#' 
+#'
 #' # U control chart from build-in data set of hospital infection rates faceted
 #' #   by hospital and type of infection.
-#' qic(month, n, 
+#' qic(month, n,
 #'     n        = days,
 #'     data     = hospital_infections,
 #'     facets   = infection ~ hospital,
@@ -97,8 +104,8 @@
 #'     title     = 'Hospital infection rates',
 #'     ylab     = 'Number of infections per 10.000 risk days',
 #'     xlab     = 'Month')
-#' 
-#' @importFrom stats median
+#'
+#' @importFrom stats median coef lm
 #' @export
 
 qic <- function(x,
@@ -110,6 +117,7 @@ qic <- function(x,
                 chart            = c('run', 'i', 'mr', 'xbar', 's', 't',
                                      'p', 'pp', 'c', 'u', 'up', 'g'),
                 agg.fun          = c('mean', 'median', 'sum', 'sd'),
+                method           = c('anhoej', 'bestbox', 'cutbox'),
                 multiply         = 1,
                 freeze           = NULL,
                 part             = NULL,
@@ -211,13 +219,14 @@ qic <- function(x,
   if (is.null(freeze) || !is.null(part)) {
     freeze <- Inf
     if(is.character(part) || is.factor(part)) {
+      part <- part[!duplicated(x)]
       part.labels <- unique(part)
       part <- match(part.labels, part)[-1] - 1
     } else {
       part <- as.integer(part)
     }
   }
-  
+
   if (is.null(exclude)) 
     exclude <- Inf
   
@@ -252,8 +261,8 @@ qic <- function(x,
   
   # Aggregate data and perform analyses
   d <- qic.agg(d, got.n, part, agg.fun, freeze, exclude, 
-               chart.fun, multiply, dots.only, chart, y.neg)
-# return(d)  
+               chart.fun, multiply, dots.only, chart, method, y.neg)
+
   # Format y for p charts
   if (missing(y.percent) & chart %in% c('p', 'pp')) {
     y.percent <- TRUE
@@ -262,7 +271,7 @@ qic <- function(x,
   if (y.percent & missing(ylab)) {
     ylab <- NULL
   }
-
+  
   # Build plot
   p <- plot.qic(d, 
                 title            = title, 
@@ -276,7 +285,6 @@ qic <- function(x,
                 scales           = scales,
                 show.labels      = show.labels,
                 show.grid        = show.grid,
-                # show.sigma.lines = show.sigma.lines,
                 decimals         = decimals,
                 flip             = flip,
                 dots.only        = dots.only,
